/******************************************************************************
  * name: Donna Gabai
  * netID: dgabai
  * precept: P99
  *
  * Description: PokerHand data type. 2 of any suit is a Wild Card.
  * Can construct a hand from standard input, format a hand for output,
  * determine if a hand is a flush or a full house.
  * Test main() sets up 5-card poker hands from standard input,
  * prints flushes and full houses (one per line).
  * 
  * Dependencies: StdIn, StdOut
  * 
  ***********************************************************/

public class PokerHandWild {
    // class constants
    private static final int HANDSIZE = 5;    // five cards per hand
    private static final char WILDCARD = '2'; // 2 is a wild card
    
    // instance variables
    private String[] hand;    // each String is one card
    private int wildCards;    // number of wild cards in this hand
    
    // constructor reads a 5-card poker hand from StdIn
    public PokerHandWild() {
        hand = new String[HANDSIZE];
        wildCards = 0;
        // input cards, count wild cards
        for (int i = 0; i < HANDSIZE; i++) {
            hand[i] = StdIn.readString();
            if (hand[i].charAt(0) == WILDCARD) 
                wildCards++;
        }
        
        // sort the hand, note Queen is higher than King! Ten is highest.
        // all 2's will always be at front of hand
        java.util.Arrays.sort(hand);
    }
    
    // is this hand a flush?
    public boolean flush() {
        // looking for suit (second character) all the same
        // use suit of first non-wild card
        char suit = hand[wildCards].charAt(1);
        for (int i = wildCards + 1; i < HANDSIZE; i++) {
            // not wild card, so check suit
            if (suit != hand[i].charAt(1)) return false;
        }
        // made it through!  ALL EQUAL!
        return true;
    }
    
    // is this hand a full house?
    public boolean fullHouse() {
        // looking for 3 matching and 2 matching
        // since hand is sorted, first and last will be different
        // first card that isn't a wildcard, no more than 4 deuces per deck
        char rank1 = hand[wildCards].charAt(0);
        int sumRank1 = 1;
        // last card
        char rank2 = hand[HANDSIZE - 1].charAt(0);
        int sumRank2 = 1;
        // special case for first and last card being equal
        if (rank1 == rank2) {
            sumRank1++;
            sumRank2++;
        }
        
        // how many other cards have that rank?
        for (int i = wildCards + 1; i < HANDSIZE - 1; i++) {
            if (hand[i].charAt(0) == rank1) sumRank1++;
            if (hand[i].charAt(0) == rank2) sumRank2++;
        }
        
        // full house the hard way
        if (sumRank1 == 3 && sumRank2 == 2) return true;
        if (sumRank1 == 2 && sumRank2 == 3) return true;
        
        // using wild cards: 3 or 4 wildcards guaranteed full house
        if (wildCards > 2) return true;

        // fewer than 3 wild cards
        // eliminate 4 of a kind
        if (sumRank1 == 4 || sumRank2 == 4) return false;
        
        // 2 wild cards - need 1 + 2 or 3 of a kind
        if (wildCards == 2) {            
            if (sumRank1 + sumRank2 == 3) return true;
            if (sumRank1 == 3) return true;
        }
        // 1 wild card
        if (wildCards == 1) {
            if (sumRank1 + sumRank2 == 4) return true;
        }
        // no full house :(
        return false;
    }
    
    // string representation of this hand sorted
    public String toString() {
        String wholeHand = "";
        for (int i = 0; i < HANDSIZE; i++)
            wholeHand = wholeHand + hand[i] + " ";
        return wholeHand;
    }
    
    // test main
    public static void main(String[] args) {
        while (!StdIn.isEmpty()) {
            PokerHandWild ph = new PokerHandWild();
            if (ph.flush()) StdOut.println("     Flush: " + ph);
            if (ph.fullHouse()) StdOut.println("Full house: " + ph);
        }
    }
}
